package org.robokind.headless.motion;

import java.io.File;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;
import javax.jms.Connection;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.ManagedService;
import org.robokind.api.common.lifecycle.ServiceLifecycleProvider;
import org.robokind.api.common.lifecycle.utils.SimpleLifecycle;
import org.robokind.api.common.osgi.lifecycle.OSGiComponent;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceLifecycle;
import org.robokind.api.common.osgi.lifecycle.ConfiguredServiceParams;
import org.robokind.api.common.services.Constants;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.lifecycle.RobotJointGroupLifecycle;
import org.robokind.api.motion.servos.ServoController;
import org.robokind.api.motion.servos.ServoRobot;
import org.robokind.api.motion.utils.RobotUtils;
import org.robokind.extern.utils.apache_commons_configuration.ConfigUtils;
import org.robokind.impl.messaging.utils.ConnectionManager;
import org.robokind.impl.motion.config.RobotConfigXMLReader;
import org.robokind.impl.motion.jointgroup.RobotJointGroupConfigXMLReader;
import org.robokind.impl.motion.lifecycle.RemoteRobotHostServiceGroup;

public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    
    private final static String ROBOT_CONFIG_FILE = "./resources/robot.xml";
    private final static Robot.Id ROBOT_ID = new Robot.Id("myRobot");
    
    @Override
    public void start(BundleContext context) throws Exception {
        theLogger.info("Connecting to Servo Robot.");
        startRobot(context, ROBOT_CONFIG_FILE);
        theLogger.info("Connection Successful.");
        Connection con = ConnectionManager.createConnection(
                "admin", "admin", "client1", "test", 
                "tcp://127.0.0.1:5672");
        new RemoteRobotHostServiceGroup(
                context, ROBOT_ID, "host", "client", 
                con, null).start();
        RobotUtils.startDefaultBlender(
                context, ROBOT_ID, theDefaultBlenderInterval);
        loadJointGroup(ROBOT_ID, context, "./resources/jointgroup.xml");
    }
    
    private final static long theDefaultBlenderInterval = 40L;
    
    private static Robot startRobot(BundleContext context, String path){
        theLogger.info("Opening Robot Config File.");
        File file = ConfigUtils.getFileSystemAdapter().openFile(path);
        if(file == null){
            return null;
        }
        ConfiguredServiceParams<Robot,Object,File> robotParams = 
                new ConfiguredServiceParams(
                        Robot.class, null, File.class, null, file, null, 
                        ServoRobot.VERSION, RobotConfigXMLReader.VERSION);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, ROBOT_ID);
        ConfiguredServiceLifecycle<Robot,?,File> robotLifecycle = 
                new ConfiguredServiceLifecycle<Robot,Object,File>(robotParams){
                    @Override
                    protected Robot create(Map dependencies){
                        Robot robot = super.create(dependencies);
                        if(robot == null){
                            return null;
                        }
                        robot.connect();
                        return robot;
                    }
                };
        OSGiComponent robotComp = 
                new OSGiComponent(
                        context, robotLifecycle, null, props, true);
        robotComp.start();
        /*theLogger.info("Loading config and initializing Robot.");
        Robot robot;
        try{
            robot = ServiceConnectionDirectory.buildService(context, 
                    ServoRobot.VERSION, RobotConfigXMLReader.VERSION, 
                    file, File.class, Robot.class);
        }catch(Exception ex){
            theLogger.log(Level.WARNING, 
                    "There was an error loading the config file.", ex);
            return null;
        }
        if(robot == null){
            theLogger.warning(
                    "Unable to load Robot.  Could not find loaders.");
            return null;
        }
        theLogger.log(Level.INFO, 
                "Connecting Robot: {0}.", robot.getRobotId());
        robot.connect();
        if(!RobotUtils.isRobotIdAvailable(context, ROBOT_ID)){
            return null;
        }
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robot.getRobotId().getRobtIdString());
        OSGiComponent<Robot> comp = new OSGiComponent<Robot>(context, 
                new SimpleLifecycle<Robot>(robot, Robot.class, props));
        comp.start();
        if(!comp.isRegistered()){
            theLogger.log(Level.INFO, "Failed registering Robot service.");
            return null;
        }
        if(robot instanceof ServoRobot){
            setServoRobot(context, (ServoRobot)robot);
        }
        return robot;*/
        return null;
    }
    
    private static void setServoRobot(BundleContext context, ServoRobot robot){
        Map<ServoController.Id,ServoController> controllers = 
                robot.getControllers();
        if(controllers == null || controllers.isEmpty()){
            return;
        }
        //TimeUtils.sleep(4000);
        for(ServoController controller : controllers.values()){
            context.registerService(ServoController.class.getName(), 
                    controller, new Properties());
        }
    }
    
    protected static ManagedService loadJointGroup(
            Robot.Id robotId, BundleContext context, String path){
        File file = ConfigUtils.getFileSystemAdapter().openFile(path);
        if(file == null){
            return null;
        }
        String paramId = "robot/" + robotId + "/jointgroup/config/param/xml";
        launchJointGroupConfig(context, file, paramId);
        return launchJointGroupLifecycle(context, robotId, paramId);
    }
    
    protected static OSGiComponent launchJointGroupLifecycle(
            BundleContext context, Robot.Id robotId, String paramId){
        RobotJointGroupLifecycle<File> lifecycle =
                new RobotJointGroupLifecycle<File>(robotId, File.class, 
                        paramId, RobotJointGroupConfigXMLReader.VERSION);
        OSGiComponent jointGroupComp = new OSGiComponent(context, lifecycle);
        jointGroupComp.start();
        return null;
    }
    
    protected static OSGiComponent launchJointGroupConfig(BundleContext context, 
            File jointGroupConfigXML, String configFileId){
        Properties props = new Properties();
        props.put(Constants.CONFIG_PARAM_ID, configFileId);
        props.put(Constants.CONFIG_FORMAT_VERSION, 
                RobotJointGroupConfigXMLReader.VERSION.toString());
        ServiceLifecycleProvider lifecycle = new SimpleLifecycle(
                        jointGroupConfigXML, File.class, props);
        OSGiComponent paramComp = new OSGiComponent(context, lifecycle);
        paramComp.start();
        return paramComp;
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        // TODO add deactivation code here
    }

}